# frozen_string_literal: true

require 'puppet_x/enterprisemodules/transformers'
require 'puppet_x/enterprisemodules/attribute_helpers'

newproperty(:absent_in) do
  include EasyType
  include PuppetX::EnterpriseModules::LdapAccess
  include PuppetX::EnterpriseModules::Transformers
  include PuppetX::EnterpriseModules::AttributeHelpers

  defaultto({})

  desc <<-DESC
    Using this property, you can manage group relations. You can make sure an LDAP entry
    is **not** in a group.

    Here is an example:

      ldap_principal { 'ldap_server:cn=muppets,ou=muppetshow,dc=example,dc=org':
        ...
        absent_in       => {
          'member'       => [
            'cn=kermit,ou=muppetshow,dc=example,dc=org',
            'cn=piggy,ou=muppetshow,dc=example,dc=org',
          ],
          ...
        },

    This puppet code makes sure that `kermit` and `piggy` are no longer part of the group
    `muppets`.

  DESC

  data_type('Optional[Hash]')

  on_modify do
    is = resource.provider.attributes
    operations = []
    removed_attributes(is, should).each do |attribute|
      operations << [:delete, attribute, should[attribute]]
    end
    ldap(resource[:ldap_server]).modify(resource[:distinguished_name], operations)
    nil
  end
  # rubocop:disable Naming/MethodParameterName
  def insync?(_is)
    # rubocop:enable Naming/MethodParameterName

    #
    # We use the value from attributes. This one is a full representation of
    # the resource actual value.
    #
    removed_attributes(resource.provider.attributes, should).empty?
  end

  # rubocop:disable Naming/MethodParameterName
  def change_to_s(is, should)
    # rubocop:enable Naming/MethodParameterName
    # rubocop:disable Lint/ShadowedArgument
    is = resource.provider.attributes
    # rubocop:enable Lint/ShadowedArgument
    message = []
    removed_attributes(is, should).each do |attribute|
      message << remove_message(attribute, is[attribute], should[attribute])
    end
    message.join(',')
  end

  # rubocop:disable Naming/MethodParameterName
  def remove_message(attribute, is, remove)
    # rubocop:enable Naming/MethodParameterName
    "Removing attribute(s) #{attribute} value(s) '#{remove}' from #{is}"
  end

  # rubocop:disable Naming/MethodParameterName
  def removed_attributes(is, should)
    # rubocop:enable Naming/MethodParameterName
    attributes = changed_attributes(value.keys, is, should)
    attributes.reject do |attribute|
      (Array(is[attribute]) - (Array(is[attribute]) - Array(should[attribute]))).empty?
    end
  end
end
