# frozen_string_literal: true

require 'base64'
require 'digest'

module PuppetX
  module EnterpriseModules
    # docs
    module AttributeHelpers
      def unsafe_munge(value)
        #
        # Make all key lowercase symbols. This is the way the net-ldap returns them
        #
        value.transform_keys { |k| k.downcase.to_sym }
      end

      def unsafe_validate(value)
        raise "Currently: #{value}, but it should be a Hash." unless value.is_a?(::Hash)
      end

      def is
        case retrieve
        when :absent then {}
        else retrieve
        end
      end

      # rubocop:disable Naming/MethodParameterName
      def changed_attributes(variables, is, should)
        # rubocop:enable Naming/MethodParameterName
        return variables if (is == :absent) || (should == :absent)

        variables.reject { |e| compare(e, is, should) }
      end

      # rubocop:disable Naming/MethodParameterName
      def updated_attributes(is, should)
        # rubocop:enable Naming/MethodParameterName
        changed_attributes(value.keys, is, should)
      end

      # rubocop:disable Naming/MethodParameterName
      def compare(attribute, is, should)
        # rubocop:enable Naming/MethodParameterName
        if should_be_transformed(attribute)
          transformed_compare(attribute, is, should)
        elsif is[attribute].is_a?(Array)
          is[attribute].sort == Array(should[attribute]).sort
        else
          is[attribute] == should[attribute]
        end
      end

      def should_be_transformed(attribute)
        if resource.transform
          resource.transform.key?(attribute.to_s)
        else
          false
        end
      end

      # rubocop:disable Naming/MethodParameterName
      def transformed_compare(attribute, is, should)
        # rubocop:enable Naming/MethodParameterName
        raise "#{attribute} should not be transformed" unless should_be_transformed(attribute)

        function_name = resource.transform[attribute.to_s]
        if is[attribute].is_a?(Array)
          compare_valid_value(function_name, should[attribute], is[attribute])
        else
          call_transformer(function_name, should[attribute], is[attribute])
        end
      end

      # rubocop:disable Naming/MethodParameterName
      def call_transformer(transformer, should, is)
        # rubocop:enable Naming/MethodParameterName
        method_name = "#{transformer}_compare".to_sym
        raise "transformer #{transformer} doesn't exist." unless respond_to?(method_name)

        send(method_name, should, is)
      end

      #
      # Calls the transformer for the values in the array of is values
      # If one comes back with a valid true or false, return this value
      # If only errors are raised by the transformer, return an error stating
      # we cannot transform the current value
      #
      # rubocop:disable Naming/MethodParameterName
      def compare_valid_value(function_name, should, is)
        # rubocop:enable Naming/MethodParameterName
        is.each do |array_value|
          value = call_transformer(function_name, should, array_value)
          return value
        rescue RuntimeError
          next
        end
        fail "The returned attribute value doesn't contain value recognised by transformer."
      end
    end
  end
end
