# frozen_string_literal: true

require 'net/ldap' if Puppet.features.net_ldap?

module PuppetX
  module EnterpriseModules
    # Docs
    class Ldap
      def initialize(args)
        host, port, admin_user, admin_password, verify_ssl, use_ssl, @base = args
        hash = {
          :host => host,
          :port => port,
          :auth => { :method => :simple, :username => admin_user, :password => admin_password }
        }
        hash[:encryption] = { :method => :simple_tls, :tls_options => { :verify_mode => OpenSSL::SSL::VERIFY_NONE } } if use_ssl && !verify_ssl
        hash[:encryption] = { :method => :simple_tls, :tls_options => {} } if use_ssl && verify_ssl

        @ldap = Net::LDAP.new(hash)
        Puppet.debug("Connecting to LDAP server ldaps://#{host}:#{port}")
        @ldap.bind
      end

      def search(args)
        args = { :base => @base }.merge(args)
        Puppet.debug("LDAP Search: #{args.inspect}")
        result = @ldap.search(args)
        code, message = return_code_and_message
        case code
        when Status::NoSuchObject
          []
        when Status::Success
          result
        else
          raise message
        end
      end

      # rubocop:disable Naming/MethodParameterName
      def add(dn, args)
        # rubocop:enable Naming/MethodParameterName
        Puppet.debug("LDAP Add: #{args.inspect}")
        report_ldap_status_for { @ldap.add(:dn => dn, :attributes => args) }
      end

      # rubocop:disable Naming/MethodParameterName
      def delete(dn)
        # rubocop:enable Naming/MethodParameterName
        Puppet.debug("LDAP Delete: #{dn}")
        report_ldap_status_for { @ldap.delete(:dn => dn) }
      end

      # rubocop:disable Naming/MethodParameterName
      def delete_attribute(dn, attribute)
        # rubocop:enable Naming/MethodParameterName
        Puppet.debug("LDAP Delete attribute #{attribute} from #{dn}")
        report_ldap_status_for { @ldap.delete_attribute(dn, attribute) }
      end

      # rubocop:disable Naming/MethodParameterName
      def delete_tree(dn)
        # rubocop:enable Naming/MethodParameterName
        Puppet.debug("LDAP Delete tree #{dn}")
        # TODO: When this functionality is added to the gem, we can remove it here.
        # Recursively delete a dn and it's subordinate children.
        # This is useful when a server does not support the DELETE_TREE control code.
        @ldap.search(:base => dn, :scope => Net::LDAP::SearchScope_SingleLevel) do |entry|
          delete_tree(entry.dn)
        end
        # Delete Self
        raise Net::LDAP::Error, get_operation_result[:error_message].to_s unless @ldap.delete(:dn => dn)

        true
      end

      # rubocop:disable Naming/MethodParameterName
      def modify(dn, operations)
        # rubocop:enable Naming/MethodParameterName
        Puppet.debug("LDAP modify #{dn} with #{operations}")
        report_ldap_status_for { @ldap.modify(:dn => dn, :operations => operations) }
      end

      private

      def report_ldap_status_for
        yield
        code, message = return_code_and_message
        raise message if code != Status::Success
      end

      def return_code_and_message
        result = @ldap.get_operation_result
        [result.code, "#{lookup_error(result.code)}: #{result.error_message}"]
      end

      def lookup_error(code)
        Status.constants.collect do |c|
          [
            Status.const_get(c), c.to_s.gsub(/([A-Z])/, ' \1').strip
          ]
        end.to_h[code]
      end

      # rubocop: disable Naming/ConstantName
      module Status
        Success                      = 0
        OperationsError              = 1
        ProtocolError                = 2
        TimeLimitExceeded            = 3
        SizeLimitExceeded            = 4
        CompareFalse                 = 5
        CompareTrue                  = 6
        StrongAuthNotSupported       = 7
        StrongAuthRequired           = 8
        OnlyPartialResultsReturned   = 9
        LdapReferral                 = 10
        AdminLimitExceeded           = 11
        UnavailableCriticalExtension = 12
        ConfidentialityRequired      = 13
        SaslBindInProgress           = 14
        NoSuchAttribute              = 16
        UndefinedAttributeType       = 17
        InappropriateMatching        = 18
        ConstraintViolation          = 19
        AttributeOrValueExists       = 20
        InvalidSyntax                = 21
        NoSuchObject                 = 32
        AliasProblem                 = 33
        InvalidDNSyntax              = 34
        ObjectIsLeaf                 = 35
        AliasDereferenceProblem      = 36
        InappropriateAuthentication  = 48
        InvalidCredentials           = 49
        InsufficientAccessRights     = 50
        Busy                         = 51
        Unavailable                  = 52
        UnwillingToPerform           = 53
        LoopDetected                 = 54
        NamingViolation              = 64
        ObjectClassViolation         = 65
        NotAllowedOnNonLeaf          = 66
        NotAllowedOnRDN              = 67
        EntryAlreadyExists           = 68
        NoObjectClassModifications   = 69
        ResultsTooLarge              = 70
        AffectsMultipleDSAs          = 71
        UnknownError                 = 80
      end
      # rubocop: enable Naming/ConstantName
    end
  end
end
